/*
Copyright © 2023 Philipp Wolfer <phw@uploadedlobster.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

package spotify

import (
	"context"
	"errors"
	"net/http"
	"strconv"
	"time"

	"github.com/go-resty/resty/v2"
	"golang.org/x/oauth2"
)

const baseURL = "https://api.spotify.com/v1/"
const MaxItemsPerGet = 50
const DefaultRateLimitWaitSeconds = 5

type Client struct {
	HttpClient *resty.Client
}

func NewClient(token oauth2.TokenSource) Client {
	ctx := context.Background()
	httpClient := oauth2.NewClient(ctx, token)
	client := resty.NewWithClient(httpClient)
	client.SetBaseURL(baseURL)
	client.SetHeader("Accept", "application/json")
	client.SetRetryCount(5)
	client.AddRetryCondition(
		func(r *resty.Response, err error) bool {
			code := r.StatusCode()
			return code == http.StatusTooManyRequests || code >= http.StatusInternalServerError
		},
	)
	client.SetRetryMaxWaitTime(time.Duration(1 * time.Minute))
	client.SetRetryAfter(func(client *resty.Client, resp *resty.Response) (time.Duration, error) {
		var err error
		var retryAfter int = DefaultRateLimitWaitSeconds
		if resp.StatusCode() == http.StatusTooManyRequests {
			retryAfter, err = strconv.Atoi(resp.Header().Get("Retry-After"))
			if err != nil {
				retryAfter = DefaultRateLimitWaitSeconds
			}
		}
		return time.Duration(retryAfter * int(time.Second)), err
	})
	return Client{
		HttpClient: client,
	}
}

func (c Client) RecentlyPlayedAfter(after time.Time, limit int) (RecentlyPlayedResult, error) {
	return c.recentlyPlayed(&after, nil, limit)
}

func (c Client) RecentlyPlayedBefore(before time.Time, limit int) (RecentlyPlayedResult, error) {
	return c.recentlyPlayed(nil, &before, limit)
}

func (c Client) recentlyPlayed(after *time.Time, before *time.Time, limit int) (result RecentlyPlayedResult, err error) {
	const path = "/me/player/recently-played"
	request := c.HttpClient.R().
		SetQueryParam("limit", strconv.Itoa(limit)).
		SetResult(&result)
	if after != nil {
		request.SetQueryParam("after", strconv.FormatInt(after.Unix(), 10))
	} else if before != nil {
		request.SetQueryParam("before", strconv.FormatInt(before.Unix(), 10))
	}
	response, err := request.Get(path)

	if response.StatusCode() != 200 {
		err = errors.New(response.String())
	}
	return
}

func (c Client) UserTracks(offset int, limit int) (result TracksResult, err error) {
	const path = "/me/tracks"
	response, err := c.HttpClient.R().
		SetQueryParams(map[string]string{
			"offset": strconv.Itoa(offset),
			"limit":  strconv.Itoa(limit),
		}).
		SetResult(&result).
		Get(path)

	if response.StatusCode() != 200 {
		err = errors.New(response.String())
	}
	return
}
